﻿using Microsoft.Win32;
using System.IO;
using System.Windows;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace RenumberDAZ
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        string strFilename = AppPath() + @"\app.xml";
        public MainWindow()
        {
            InitializeComponent();
        }

        private void WriteXml()
        {
            clsXml objClass = new clsXml();
            objClass.Prefix = txtPrefix.Text;
            objClass.SceneFolder = lblDir1.Text;
            objClass.CapFolder = lblDir2.Text;
            objClass.DazCapFolder = lblDir3.Text;

            var serializer = new System.Xml.Serialization.XmlSerializer(typeof(clsXml));
            var sw = new StreamWriter(strFilename, false, new System.Text.UTF8Encoding(false));
            serializer.Serialize(sw, objClass);
            sw.Close();
        }

        private int GetNumber(string strName)
        {
            return (int.Parse(strName.Substring(txtPrefix.Text.Length + 1, 4)));
        }

        private void NextImage()
        {
            int intNum = GetNumber(System.IO.Path.GetFileNameWithoutExtension(lblFrom.Text));
            for (int i = 1; i <= 900; i++)
            {
                if (File.Exists(@$"{lblDir1.Text}\{EFN(intNum + i * 10, "duf.png")}"))
                {
                    lblTo.Text = @$"{EFN(intNum + i * 10, "duf.png")}";
                    picTo.ImageSource = LoadImage(@$"{lblDir1.Text}\{EFN(intNum + i * 10, "duf.png")}");
                    break;
                }
            }
        }

        private BitmapImage LoadImage(string strPath) 
        {
            BitmapImage image = new BitmapImage();
            image.BeginInit();
            image.CacheOption = BitmapCacheOption.OnLoad;
            image.UriSource = new Uri(strPath);
            image.EndInit();
            return image;
        }

        private void ReNumber()
        {
            int intGyo = 0;

            lblMsg.Content = "";
            if (int.TryParse(txtGyo.Text, out intGyo))
            {
                intGyo = int.Parse(txtGyo.Text);
            }
            else
            {
                txtGyo.Text = "0";
            }
            if (intGyo == 0)
            {
                lblMsg.Content ="Enter the number of rows";
                return;
            }
            if (!File.Exists(@$"{lblDir1.Text}\{lblFrom.Text}")||
                !File.Exists(@$"{lblDir1.Text}\{lblTo.Text}") ||
                lblFrom.Text == lblTo.Text)
            {
                lblMsg.Content ="The range is incorrectly specified.";
                return;
            }
            if (!Directory.Exists(lblDir1.Text) ||
                !Directory.Exists(lblDir2.Text) ||
                !Directory.Exists(lblDir3.Text))
            {
                lblMsg.Content = "Scene or Cap or Daz Cap folder is not specified.";
                return;
            }
            if (lblDir1.Text == lblDir2.Text ||
                lblDir1.Text == lblDir3.Text ||
                lblDir2.Text == lblDir3.Text)
            {
                lblMsg.Content = "The same is specified in the folder.";
                return;
            }
            string strRtn = CheckExist();
            if (strRtn != "")
            {
                lblMsg.Content = $"Cannot renumber because {strRtn} exists.";
                return;
            }
            if (MessageBox.Show("Is it all right if I renumber?", this.Title, MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.No)
                return;


            try
            {
                if (intGyo > 0)
                {
                    for (int i = 9990 - intGyo * 10; i >= GetNumber(lblTo.Text); i-=10)
                    {
                        Rename(i, intGyo);
                    }
                }
                else
                {
                    for (int i = GetNumber(lblTo.Text); i <= 9990; i+=10)
                    {
                        Rename(i, intGyo);
                    }
                }

                lblMsg.Content = "The process was successfully completed.";
                NextImage();

            }
            catch (Exception ex)
            {
                lblMsg.Content = ex.Message;
            }

        }

        private void Rename(int i, int intGyo)
        {
            //Rename Daz file in Scenes
            if (File.Exists(@$"{lblDir1.Text}\{EFN(i, "duf")}"))
                Directory.Move(@$"{lblDir1.Text}\{EFN(i, "duf")}", @$"{lblDir1.Text}\{EFN(i + intGyo * 10, "duf")}");
            if (File.Exists(@$"{lblDir1.Text}\{EFN(i, "duf.png")}"))
                Directory.Move(@$"{lblDir1.Text}\{EFN(i, "duf.png")}", @$"{lblDir1.Text}\{EFN(i + intGyo * 10, "duf.png")}");
            if (File.Exists(@$"{lblDir1.Text}\{EFN(i, "tip.png")}"))
                Directory.Move(@$"{lblDir1.Text}\{EFN(i, "tip.png")}", @$"{lblDir1.Text}\{EFN(i + intGyo * 10, "tip.png")}");
            //Rename png, jpg, psd, and clip files in each storage directory in Caps
            if (File.Exists(@$"{lblDir2.Text}\{EFN(i, "png")}"))
                Directory.Move(@$"{lblDir2.Text}\{EFN(i, "png")}", @$"{lblDir2.Text}\{EFN(i + intGyo * 10, "png")}");
            if (File.Exists(@$"{lblDir2.Text}\{EFN(i, "jpg")}"))
                Directory.Move(@$"{lblDir2.Text}\{EFN(i, "jpg")}", @$"{lblDir2.Text}\{EFN(i + intGyo * 10, "jpg")}");
            if (File.Exists(@$"{lblDir2.Text}\{EFN(i, "psd")}"))
                Directory.Move(@$"{lblDir2.Text}\{EFN(i, "psd")}", @$"{lblDir2.Text}\{EFN(i + intGyo * 10, "psd")}");
            if (File.Exists(@$"{lblDir2.Text}\{EFN(i, "clip")}"))
                Directory.Move(@$"{lblDir2.Text}\{EFN(i, "clip")}", @$"{lblDir2.Text}\{EFN(i + intGyo * 10, "clip")}");
            //Rename png, jpg, psd, and clip files in Caps
            if (File.Exists(@$"{lblDir3.Text}\{EFN(i, "png")}"))
                Directory.Move(@$"{lblDir3.Text}\{EFN(i, "png")}", @$"{lblDir3.Text}\{EFN(i + intGyo * 10, "png")}");
            if (File.Exists(@$"{lblDir3.Text}\{EFN(i, "jpg")}"))
                Directory.Move(@$"{lblDir3.Text}\{EFN(i, "jpg")}", @$"{lblDir3.Text}\{EFN(i + intGyo * 10, "jpg")}");
            if (File.Exists(@$"{lblDir3.Text}\{EFN(i, "psd")}"))
                Directory.Move(@$"{lblDir3.Text}\{EFN(i, "psd")}", @$"{lblDir3.Text}\{EFN(i + intGyo * 10, "psd")}");
            if (File.Exists(@$"{lblDir3.Text}\{EFN(i, "clip")}"))
                Directory.Move(@$"{lblDir3.Text}\{EFN(i, "clip")}", @$"{lblDir3.Text}\{EFN(i + intGyo * 10, "clip")}");
        }

        private string CheckExist()
        {
            try
            {
                int intGyo = 0;
                int k = 0;
                if (int.TryParse(txtGyo.Text, out intGyo))
                {
                    intGyo = int.Parse(txtGyo.Text);
                }
                if (intGyo == 0) return ("");

                if (intGyo < 0)
                {
                    for (int i = GetNumber(lblTo.Text); i <= 9990; i+=10)
                    {
                        k--;
                        if (k < intGyo) break;
                        if (File.Exists(@$"{lblDir1.Text}\{EFN(i + intGyo * 10, "duf")}"))
                            return (@$"{lblDir1.Text}\{EFN(i + intGyo * 10, "duf")}");
                        if (File.Exists(@$"{lblDir1.Text}\{EFN(i + intGyo * 10, "duf.png")}"))
                            return (@$"{lblDir1.Text}\{EFN(i + intGyo * 10, "duf.png")}");
                        if (File.Exists(@$"{lblDir1.Text}\{EFN(i + intGyo * 10, "tip.png")}"))

                            return (@$"{lblDir1.Text}\{EFN(i + intGyo * 10, "tip.png")}");
                        if (File.Exists(@$"{lblDir2.Text}\{EFN(i + intGyo * 10, "png")}"))
                            return (@$"{lblDir2.Text}\{EFN(i + intGyo * 10, "png")}");
                        if (File.Exists(@$"{lblDir2.Text}\{EFN(i + intGyo * 10, "jpg")}"))
                            return (@$"{lblDir2.Text}\{EFN(i + intGyo * 10, "jpg")}");
                        if (File.Exists(@$"{lblDir2.Text}\{EFN(i + intGyo * 10, "psd")}"))
                            return (@$"{lblDir2.Text}\{EFN(i + intGyo * 10, "psd")}");
                        if (File.Exists(@$"{lblDir2.Text}\{EFN(i + intGyo * 10, "clip")}"))
                            return (@$"{lblDir2.Text}\{EFN(i + intGyo * 10, "clip")}");

                        if (File.Exists(@$"{lblDir3.Text}\{EFN(i + intGyo * 10, "png")}"))
                            return (@$"{lblDir3.Text}\{EFN(i + intGyo * 10, "png")}");
                        if (File.Exists(@$"{lblDir3.Text}\{EFN(i + intGyo * 10, "jpg")}"))
                            return (@$"{lblDir3.Text}\{EFN(i + intGyo * 10, "jpg")}");
                        if (File.Exists(@$"{lblDir3.Text}\{EFN(i + intGyo * 10, "psd")}"))
                            return (@$"{lblDir3.Text}\{EFN(i + intGyo * 10, "psd")}");
                        if (File.Exists(@$"{lblDir3.Text}\{EFN(i + intGyo * 10, "clip")}"))
                            return (@$"{lblDir3.Text}\{EFN(i + intGyo * 10, "clip")}");
                    }
                }
                return ("");
            }
            catch (Exception ex)
            {
                lblMsg.Content = ex.Message;
                return ("");
            }
        }

        private string EFN(int intNum, string strId)
        {
            return ($"{txtPrefix.Text}-{intNum.ToString("0000")}.{strId}");
        }

        private static string AppPath()
        {
            return (System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetExecutingAssembly().Location));
        }

        private string OpenFolder(string strTitle, string strInitFolder)
        {
            OpenFolderDialog dialog = new OpenFolderDialog();
            dialog.Title = strTitle;
            dialog.DefaultDirectory = strInitFolder;

            if (dialog.ShowDialog(Owner) == true)
            {
                return dialog.FolderName;
            }
            else return ("");
        }
        private string OpenFile(string strFolder, string strTitle, string strInitFile)
        {
            OpenFileDialog dialog = new OpenFileDialog();
            dialog.FileName = strInitFile;
            dialog.InitialDirectory = strFolder;
            dialog.FilterIndex = 2;
            dialog.Title = strTitle;
            dialog.RestoreDirectory = true;
            dialog.CheckFileExists = true;
            dialog.CheckPathExists = true;
            dialog.Filter = $"Pingファイル({txtPrefix.Text}*.duf.png)|{txtPrefix.Text}*.duf.png";
            if (dialog.ShowDialog() == true) return (dialog.FileName);
            return ("");
        }

        private void btnOpen1_Click(object sender, RoutedEventArgs e)
        {
            string strRtn = OpenFolder("Select a Scene folder", lblDir1.Text);
            if (strRtn != "") lblDir1.Text=strRtn;
        }

        private void btnOpen2_Click(object sender, RoutedEventArgs e)
        {
            string strRtn = OpenFolder("Select a Cap folder", lblDir2.Text);
            if (strRtn != "") lblDir2.Text=strRtn;
        }

        private void btnOpen3_Click(object sender, RoutedEventArgs e)
        {
            string strRtn = OpenFolder("Select a DAZ Cap folder", lblDir3.Text);
            if (strRtn != "") lblDir3.Text=strRtn;
        }

        private void Window_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            WriteXml();
        }

        private void picFrom_Click(object sender, RoutedEventArgs e)
        {
            lblMsg.Content = "";
            if (lblDir1.Text == "" &&  lblDir2.Text == "" && lblDir3.Text == "")
            {
                lblMsg.Content = "Scene or Cap or Daz Cap folder is not specified.";
                return;
            }

            if (!Directory.Exists(lblDir1.Text) ||
                !Directory.Exists(lblDir2.Text) ||
                !Directory.Exists(lblDir3.Text))
            {
                lblMsg.Content = "Scene or Cap or Daz Cap folder is not specified.";
                return;
            }

            if (txtPrefix.Text.Trim() == "")
            {
                lblMsg.Content = "No prefix entered.";
                return;
            }

            string strRtn = OpenFile(lblDir1.Text, "Select a starting image.", lblFrom.Text);
            if (strRtn != "")
            {
                lblFrom.Text = System.IO.Path.GetFileName(strRtn);
                picFrom.ImageSource = LoadImage(strRtn);

                NextImage();
                txtGyo.Focus();
            }
        }

        private void btnOk_Click(object sender, RoutedEventArgs e)
        {
            ReNumber();
        }

        private void txtGyo_GotFocus(object sender, RoutedEventArgs e)
        {
            txtGyo.SelectAll();
        }

        private void txtPrefix_GotFocus(object sender, RoutedEventArgs e)
        {
            txtPrefix.SelectAll();
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            try
            {
                if (!File.Exists(strFilename)) WriteXml();

                var serializer = new System.Xml.Serialization.XmlSerializer(typeof(clsXml));
                var sr = new StreamReader(strFilename, new System.Text.UTF8Encoding(false));
                clsXml objClass = (clsXml)serializer.Deserialize(sr);
                if (objClass != null)
                {
                    txtPrefix.Text = objClass.Prefix;
                    lblDir1.Text = objClass.SceneFolder;
                    lblDir2.Text = objClass.CapFolder;
                    lblDir3.Text = objClass.DazCapFolder;
                }
                sr.Close();

                btnOk.IsEnabled = true;
                btnOpen1.IsEnabled = true;
                btnOpen2.IsEnabled = true;
                btnOpen3.IsEnabled = true;
                btnFrom.IsEnabled = true;
                txtGyo.IsEnabled = true;
                txtPrefix.IsEnabled = true;
            }
            catch (Exception ex)
            {
                lblMsg.Content = ex.Message;
            }
        }
    }
    public class clsXml
    {
        public string Prefix = "";
        public string SceneFolder = "";
        public string CapFolder = "";
        public string DazCapFolder = "";
    }
}